//  ASL RUN TIME SYSTEM LIBRARY
//
//  ASL (Abstract Schema Language) to C++ Compiler
//
//
//
//  FILE NAME   : asl_seq.h
//  MODULE TYPE : ASL Runtime System Library
//  STATUS      : In development
//  PURPOSE     : Export definition of the ASL runtime library
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1995  
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.0 (Apr 20, 1992 - Longguang Wu),
//            Start of development.
//
//            0.5 (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 Alpha.
//
//            0.7 (Oct 28, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.0 (Aug 12, 1996 - Claudia Calderas & Salvador Marmol), 
//            Version 1.0 Beta.
//
//            1.0 (Oct 23, 1997 - Oscar Peguero), 
//            Version 1.0 Beta 2.
//
//            1.1 (Ene 29, 1998 - Oscar Peguero), 
//            Simple version for PC with double buffering (no threads).


#pragma interface

#ifndef __ASL_SEQ_H
#define __ASL_SEQ_H 1

#ifndef __ASL_PVM_H
#define __ASL_PVM_H 1
#include "/opt/local/pvm3/include/pvm3.h"
#endif

//class AslMutex;
//AslMutex MutexExec;

class AslState
{
	AslSchema *sch_obj;
	int state;
	int steps;
	int thr_flag;
	
	AslMutex *stateSync;
	AslMutex *stateSync2;
	AslCondition *stateCond;
	AslCondition *stateCond2;
//	AslListSchema depList; // AslList<AslSchema>
public:
	AslState(void) { state = READY; steps = 0; sch_obj = NULL; stateSync = new AslMutex; stateSync2 = new AslMutex; thr_flag = 0;
			stateCond = new AslCondition; stateCond2 = new AslCondition; }
	~AslState(void) { delete stateSync; delete stateSync2; delete stateCond; delete stateCond2;}
	
	void setObj(AslSchema *); //sch_obj) { obj = sch_obj; aslio<< "Setting AslState Object" << aslend; }
	void setState(int); // stat) { state = stat; aslio << "Setting AslState state:" << stat << aslend; }
	void setStep(int); // step) { steps = step; aslio << "Setting AslState steps:" << step << aslend; }
	void incStep(void);
	void decStep(void);
	void setThrFlag(void);
	void unsetThrFlag(void);	

	AslSchema* getObj(void) { return sch_obj; }
	int getState(void) { return state; }
	int getStep(void) { return steps; }
	AslMutex* getStateSync(void) { return stateSync; }
	AslMutex* getStateSync2(void) { return stateSync2; }
//	AslCondition getStateCond(void) { return &stateCond; }
	void wait(void);
	void wait2(void) { stateCond2->AslConditionWait(stateSync2); }
	void signal2(void) { stateCond2->AslConditionSignal(); }
	void signal(void);
	int haveThread(void);
		
};

class AslSchema_Pvm
{
	AslSchema *obj;
	int	gpid;
public:
	AslSchema_Pvm(void) { gpid = -1; obj = NULL;}
	~AslSchema_Pvm(void) {}
	
	AslSchema* getObj(void) { return obj; }
	void setObj(AslSchema *sch_obj) { obj = sch_obj; /*aslio << "Setting AslSchema_Pvm Object:" << aslend;*/ }
	int getGpid(void) { return gpid; }
	void setGpid(int id) { gpid = id; /*aslio << "Setting AslSchema_Pvm gpid:" << id << aslend;*/ }
};

/* struct aslArgument
{
	int argc;
	char **argv;
};	*/

// Structure for connections y relabels

class AslPortRelations
{
//	char noentiendo[100];
	int port_id;	// now is a port ID/ PVM Msgtag  !!antes bid !!
	int proc_id;	// PVM tid. i.e. gpid
public:
	AslPortRelations(void) { port_id = -1; proc_id = -1;}
	~AslPortRelations(void) {}
	int getPort_id(void) { return port_id; }
	void setPort_id(int id) { port_id = id; /*aslio << "Port_id:" << port_id << aslend;*/ }
	int getProc_id(void) { return proc_id; }
	void setProc_id(int id) { proc_id = id; /*aslio << "Proc_id:" << proc_id << aslend;*/ }
};

class AslSystem : public nsl_public
{
	AslList<AslSchema_Pvm> schemaList;	// Container schema list
	AslList<AslState>  schemaExecList;
	
	int breakfg; // -1 sim, -2 epoch
	
	int proc_num;	// number of PVM processes+
	int *gpids;	// pvm-process TID's
	char **schema_buf;	// for storing schema names relating to gpids
	
	AslMutex	MutexExec;
	AslMutex	MutexSchList;
//	static AslCondition	schedCond;
//	static AslMutex		*schedMutex;
	
//	int cont;
	
//	aslArgument arg;
	
	char fileName[60];
public:
	AslSystem(int, char **);
	~AslSystem();
	
	void set_breakfg(int fg) { breakfg = fg; }
	int get_breakfg() { return breakfg; }
	
	void AslAddSchema(AslSchema*);
	void AslRemoveSchema(char*);
//	void AslRemExecSchema(char *);
	
//	void AslGraphInit(aslArgument);
	void AslGeneralScheduler(void);
	
// new execution
	int AslDoListMakeConn(void);
	int AslDoListInitModule(void);
	int AslDoListEndModule(void);
	int AslDoListInitSys(void);
	int AslDoListEndSys(void);
	int AslDoListInitRun(void);
	int AslDoListSimRun(void);
	int AslDoListEndRun(void);
	int AslDoListInitTrain(void);
	int AslDoListSimTrain(void);
	int AslDoListEndTrain(void);

	void AslDoListConnect(void);
	void AslDoListInit(void);
	void AslDoListRun(void);

// old execution
	void AslDoListStep(void);
	void AslExecMethod(AslState*);
	
//	void AslParseArguments(int, char**);
	
	AslSchema_Pvm* AslGetSchema(char*);	DIFERENTE EN NSL !!
	
	void AslLoadFile(void);
	
	AslList<AslState>& getExecList(void) { return schemaExecList; };
	AslList<AslSchema_Pvm>& getSchemaList(void) { return schemaList; };	
	
	AslMutex& getMutexExec(void) { return MutexExec; }
	
// for interaction with console
	void AslWaitConsoleTids(void);
	int AslLocalExecution(char *);	
	int AslGetGpidBuf(int i) { return gpids[i]; }
	char *AslGetSchemaBuf(int i) { return schema_buf[i]; }
	int AslGetProcNum(void) { return proc_num; }
	
// for graphic data handling
	void AslAcceptGraphData(char *);
	void AslSendGraphData(char *);
	
// for threading schema functions
	static void* FuncThread(void*);
	static void* FuncThread2(void*);	
	static void* FuncThread3(void*);
	static void* FuncScheduler(void*);
	
// for Initializing schema's function scheduler thread
	void InitFuncThread(void);
	void CreateFuncSched(AslState *);
	
// for Killing schema's function scheduler thread
	void FinishFuncThread(void);
};

// Class: AslSchemaInterface

class AslSchemaInterface : public nsl_model
{
protected:

	AslList<AslPort> *aslExternalInputPortList;      // External input port list
	AslList<AslPort> *aslExternalOutputPortList;     // External output port list

	AslList<AslFunction>   *aslExternalFuncList;     // External function list

	AslFunction* AslGetExternalFunc(char*);

public:
	AslSchemaInterface(char * = "" , nsl_model* = 0, int = 1);
	~AslSchemaInterface(void);

	AslPort* AslGetExternalInputPort(char*, int i);
	void AslAddExternalInputPort(AslPort*);

	AslPort* AslGetExternalOutputPort(char*, int i);
	void AslAddExternalOutputPort(AslPort*);
/*
	AslPort* AslGetExternalInOutPort(char*, int i);
	void AslAddExternalInOutPort(AslPort*);
*/
	void AslAddExternalFunc(AslFunction*);
//	asl_any_t AslExecuteRPC(char *, asl_any_t arg);
};

// Class: AslSchema

class AslSchema : public AslSchemaInterface
{
	int tid;

protected:

	AslList<AslPort>  *aslInternalInputPortList;   // Internal input port list
	AslList<AslPort> *aslInternalOutputPortList;   // Internal output port list

//	AslList<AslPort>  *aslInternalInOutPortList;   // Internal input/output port list

	char name[256];
	
	int steps;	// For maintaining schema run method steps

//	AslSystem *aslSystemPtr;
//	AslInOutPort<nsl_layer, NSL_LAYER>	*GraphPort;	// For passing graphic remote data

public:
	AslSchema(char * = "" , nsl_model* = 0, int = 1);	
	~AslSchema(void);

	AslPort* AslGetInternalInputPort(char*, int);
	void AslAddInternalInputPort(AslPort*);

	AslPort* AslGetInternalOutputPort(char*, int);
	void AslAddInternalOutputPort(AslPort*);

	int AslSearchInternalInputPortOcurrence(char *);
	int AslSearchExternalInputPortOcurrence(char *);	
	int AslSearchInternalOutputPortOcurrence(char *);
	int AslSearchExternalOutputPortOcurrence(char *);
/*
	int AslSearchExternalInOutPortOcurrence(char *);	
	int AslSearchInternalInOutPortOcurrence(char *);

	AslPort* AslGetInternalInOutPort(char*, int);
	void AslAddInternalInOutPort(AslPort*);
*/
	int nslBreakSim();
	int nslBreakEpoch();

	// added 10-98 by Alfredo
	void connect(nsl_layer&, nsl_layer&);	
	void connect(AslPort *, AslPort *);
	void connect(AslPort *, int, int);		// for connecting the remote graphic variable

	void disconnect(AslPort *, AslPort *);
	void disconnect(AslPort *, int, int);

	void relabel(nsl_layer&, nsl_layer&);	
	
	
	void relabel(AslPort*, AslPort*);	// antes en AslOutputPort !!!!!	
	void delabel(AslPort*, AslPort*);	// 	"	"
	
	void send();
	void receive();
	
//	void delegate(...);
	void delegate(AslSchema*);
	int display(void);
	void nslGraphics(AslPort *);		// for displaying the remote variable
	
	void updateBuffers(void) { }	
	void memAlloc(void) { }	
	
	virtual void makeConn(void) { }	
	virtual void initTrain(void) { }	
	virtual void simTrain(void) { }	
	virtual void endTrain(void) { }	
	virtual void initRun(void) { }	
	virtual void simRun(void) { }	
	virtual void endRun(void) { }	
	virtual void initSys(void) { }	
	virtual void endSys(void) { }	
	virtual void initModule(void) { }	
	virtual void endModule(void) { }	
	
	virtual void makeconn(void) {}	
	virtual void makedisconn(void) {}	
	virtual void initrun(void) {}	
	virtual void simrun(void) {}	
	
	int getSteps(void) { return steps; }
	void setSteps(int s) { steps = s; }
	char* getname(void){ return name; }
};

template<class T>
class AslSchemaModel : public nsl_model
{
public:
	AslSchemaModel(const char* str) : nsl_model(str,0,-1) {}
	
	~AslSchemaModel() {}
	
	nsl_model* makeInstance(){
		return new T;
	}
};

// Class AslFunction

class AslFunction
{
	char name[ASL_NAME_LEN];
	int argType;
	int answerType;

	AslSchema *schema;
	
//	asl_any_t answer;
public:
	AslFunction(AslSchema *, char*, int, int /*, asl_thread_func_t */);
	~AslFunction(void) {}

//	asl_thread_func_t funcPtr;
	
	char* AslGetName(void);
};

// Class AslPort 

class AslPort : public nsl_public
{
protected:

	AslList<AslPortRelations> connections;
	AslList<AslPortRelations> relabels;

	int numberOfMessages;
	int numberOfConnections;
	int numberOfRelabels;	

	char  name[ASL_NAME_LEN];      // port name
	int   comType;                 // 1 - input, 2 - output	
	int   scope;                   // 1 - external, 2 - internal
	int   index;		       // Ocurrence of the same port name
	int   bid;		       // Port ID
	int   LOCAL;
//	mutex_t m;
	
	AslSchema     *schema;
	AslIPCManager *aslComm; 

public:

	AslPort(AslSchema *, AslIPCManager *, char *, int, int);	// constructor para diferentes implementaciones de puerto
	AslPort(const char *, nsl_public *, int);

	~AslPort(void);

	int   AslGetType(void); 
	void  AslSetType(int);
	int   AslGetIndex(void); 
	int   AslGetScope(void);
	int   AslGetBid(void);
	char* AslGetName(void);
	void  AslsetSchema(AslSchema *);	// schema reference setting
	AslSchema*  AslGetSchemaAttr(void) { return schema; }
	AslIPCManager* AslGetAslCommAttr(void) { return aslComm; }
	
	int AslSearchPortOcurrence(const char*);
		
	int AslAddConnection(int, int);
	int AslDeleteConnection(int, int);
	
	int AslAddConnection(int bidc) { return AslAddConnection(bidc, AslCom->gpid); }
	int AslDeleteConnection(int bidc) { return AslDeleteConnection(bidc, AslCom->gpid); }
	
	int AslAddRelabel(int bidc);
	int AslDeleteRelabel(int bidc);
	
	virtual void send();
	// DEPENDE DE QUE IMPLEMENTACION DE PUERTO SE USE !
	virtual void receive();
// VERIFICAR QUE HACEN ESTAS FUNCIONES !!!
	
	AslPortRelations* AslGetConnection(void);
	void AslPutConnectionListAtBegining(void);
	void AslGoToNextConnection(void);
	
	AslPortRelations* AslGetRelabel(void);
	void AslPutRelabelListAtBegining(void);
	void AslGoToNextRelabel(void);
	
	void AslPrintConnects(void) {
		AslPortRelations *temp;
	
		connections.startOnTop();
		while ((temp=connections.getActual())!=NULL)
		{
			aslio << "--------Puerto:"<< bid << " -->" << temp->getPort_id() << "," << temp->getProc_id()
			<< aslend;
			connections.goNext();
		}
		if (temp==NULL)
		{
			aslio << "--------Puerto:" << bid << " NO TIENE MAS CONEXIONES" << aslend;
		}
	}
		
	int AslGetNumberOfElements(void);
	int AslIsAnInputBuffer(void);
	int AslIsAnOutputBuffer(void);
//	virtual void AslInsertData(nsl_num_matrix*) {}	// antes nsl_num_matrix
//	virtual nsl_num_matrix* AslGetVar(void) { return NULL; } // igual que arriba

	virtual void AslCopyData(AslPort*) {}	// antes nsl_num_matrix

	void setLocalFlag(void) { LOCAL = 1; }
	virtual void AslPVMSend(int) {}
	virtual void AslPVMReceive(void) {}
	
//	virtual void receive(void) {}
//	virtual void send(void) {}
};

// -----------------------------------------------------------------------------

template<class T,int t>
class AslInOutPort : public AslPort
{
public:
	T *var;
	int Type_ID;  		

	AslInOutPort(const char *str, nsl_public *s, T *var_asoc, int com_type) : 
		AslPort(str, s, com_type)
	{
		var = var_asoc;
		Type_ID = t;
	}

	~AslInOutPort(void) {}

	AslInOutPort<T>& operator>>(T &msg);	// p ? x: input, now is p >> x
	AslInOutPort<T>& operator<<(T &msg);	// P ! x: output, now is p << x	
	void AslCopyData(AslPort *);
	T* AslGetVar(void) { return var; }
	void send();
	void receive();
        void AslPVMReceive(void);
	void AslPVMSend(int);
	
	
/*  ///////////	
	AslInOutPort& operator>>(T &msg);	// p ? x: input, now is p >> x
	AslInOutPort& operator<<(T &msg);	// p ! x: output, now is p << x

	void receive(void);
*/	
};

template<class T, int t> 
void AslInOutPort<T, t>::send()
{
	*this << *var;	
}

template<class T,int t> void AslInOutPort<T,t>::receive()
{
	int res;

	if(!LOCAL)
	{
		res = aslComm->AslRecv(bid); 
	}

	*this >> *var;
}

template<class T,int t> AslInOutPort<T,t>& AslInOutPort<T,t>::operator>>(T &msg)
{
//Se esta copiando directo al puerto de nsl con el send mediante DataCopy
//AW99	msg = *var;	// recibo en var, Esto seria FilterBuf
	
	numberOfMessages--;
	
	send(); // in case they are being relabeled
	
	return(*this);
}
template<class T,int t> AslInOutPort<T,t>& AslInOutPort<T,t>::operator<<(T &msg)
{
	var = &msg;	// Esto seria en el FilterBuf
	
	numberOfMessages++;

	aslComm->AslSend(bid);
	
	numberOfMessages--;

	return(*this);	
}

template <class T, int t> 
void AslInOutPort<T, t>::AslCopyData(AslPort *DataPort)
{
	var = ((AslInOutPort<T,t>*)DataPort)->AslGetVar();
	numberOfMessages++;
}
/*	///////////////////////////////////
template <class T,int t> void AslInOutPort<T,t>::AslCopyData(AslPort *port)
{
	*var = *((AslInOutPort<T>*) port)->AslGetVar(); // copy or ref
	numberOfMessages++;
//	aslComm->AslSend(bid); //send(); // AW99 added for relabeling purposes
}
//////////////	*/
// -------------------------------------------------------------------------------
template <class T, int t>
void AslInOutPort<T,t>::AslPVMSend(int opc_xdr) 
{
//	AslPort *falla = AslCom->AslGetIOPort(8);
	
//	falla->AslPrintConnects();
//	AslPrintConnects();
	int i,j;
	
	float	r_float[121];		// var.getsize()
/*	char	*r_char;
	int	*r_int;
	long	*r_long;
	short	*r_short;
	double	*r_double; */
	
//	AslPrintConnects();
//	falla->AslPrintConnects();
			
//        if (opc_xdr) 
//                pvm_initsend(PvmDataDefault);
//        else
//                pvm_initsend(PvmDataRaw);

        switch (Type_ID)
        {
//                case ASL_CHAR: pvm_pkstr(r_char);
//                           break;

/*                case ASL_INT : for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			r_int[i*11+j] = var->elem(i,j);
                        			
				AslPrintConnects();                        			
                        			
                		pvm_pkint(r_int, 121, var.getvec(), var.get_size(), 1);
		                break;
*/		                

                case ASL_VECTOR:
                       		for(i=0; i<11; i++)
                       			r_float[i] = ((nsl_vector *)var)->elem(i);
                        			
                		pvm_pkfloat(r_float, 11, 1);
	                        break;

                case ASL_FLOAT: 
//                		AslPrintConnects();
//                		falla->AslPrintConnects();
                		
// 				r_float = new float[11*11];	//var.get_size();
	
//                     		AslPrintConnects();
//                     		falla->AslPrintConnects();
                       	
                       		for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			r_float[i*11+j] = ((nsl_matrix *)var)->elem(i,j);
                        			
//                        	AslPrintConnects();
//                        	falla->AslPrintConnects();
                        	
//			        if (opc_xdr) 
//			                pvm_initsend(PvmDataDefault);
//			        else
//			                pvm_initsend(PvmDataRaw);
			                
//			        AslPrintConnects();
//			        falla->AslPrintConnects();
                        			
                		pvm_pkfloat(r_float, 121, 1);
	                        break;
/*
                case ASL_VECTOR:
                       	
                       		for(i=0; i<11; i++)
                       			r_float[i] = var->elem(i);
                        			
                		pvm_pkfloat(r_float, 11, 1);
	                        break;
*/	                        

/*                case ASL_DOUBLE: for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			r_double[i*11+j] = var->elem(i,j);

                		pvm_pkdouble(r_double, 121, 1);
                
                             break;
                             
                case ASL_LONG:  for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			r_long[i*11+j] = var->elem(i,j);
                        			
                		pvm_pklong(r_long, 121, 1);
                		break;
                             
                case ASL_SHORT: for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			r_short[i*11+j] = var->elem(i,j);
                        			
                		pvm_pkshort(r_short, 121, 1);
                             	break; */
        }
//	AslPrintConnects();
//	falla->AslPrintConnects();
}

template <class T, int t>
void AslInOutPort<T, t>::AslPVMReceive(void) 
{
	int i,j;
	
	float r_float[121];		//var.get_size();
/*	r_char = new char[11*11];	//var.get_size();
	r_int = new int[11*11];		//var.get_size();
	r_long = new long[11*11];	//var.get_size();
	r_short = new short[11*11];	//var.get_size();
	r_double = new double[11*11];	//var.get_size();	*/
	
        switch (Type_ID)
        {
//                case ASL_CHAR: pvm_upkstr(r_char);
//                		var = r_char;
//                           	break;

/*                case ASL_INT : pvm_upkint(r_int, /* var.get_v(), var.get_size() 121, 1);
			       	for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			var->set_value(i,j, r_int[i*11+j]);
				break;
*/
                case ASL_FLOAT: pvm_upkfloat(r_float, 121, 1);
			       	for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			((nsl_matrix *)var)->set_value(i,j, r_float[i*11+j]);
                
//                		var = r_float;
                            	break;

                case ASL_VECTOR: 
                		 pvm_upkfloat(r_float, 11, 1);
			       	 for(i=0; i<11; i++)
                	       		((nsl_vector *)var)->set_value(i, r_float[i]);
                
                            	 break;


/*                case ASL_DOUBLE: pvm_upkdouble(r_double, 121, 1);
			       	for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			var->set_value(i,j, r_double[i*11+j]);
                
//                		var = r_double;
                             	break;
                             
                case ASL_LONG: pvm_upklong(r_long, 121, 1);
			       	for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			var->set_value(i,j, r_long[i*11+j]);
                
//                		var = r_long;
                             	break;
                             
                case ASL_SHORT: pvm_upkshort(r_short, 121, 1);
			       	for(i=0; i<11; i++)
                	       		for(j=0; j<11; j++)
                        			var->set_value(i,j, r_short[i*11+j]);
                
//                             	var = r_short;
                             	break; */
        }
}

#endif
