
//  ASL THREAD LIBRARY 
//
//  ASL (Abstract Schema Language) System Execution Kernel
//
//
//
//  FILE NAME   : aslthr.h
//  MODULE TYPE : ASL Thread Library File
//  STATUS      : In development 
//  PURPOSE     : Schemas execution on a multithread environment
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996-1998
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Aug 7, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Aug 11, 1996 - Claudia Calderas & Salvador Marmol), 
//            Creation of thread interface.



#ifndef __ASLTHR_H
#define __ASLTHR_H

// #include <asllist.h>

//  This code denpeds on the thread library that you want to use

//#include <thread.h>			// Include Sun thread library
//#include <sys/types.h>			// Include using types

//typedef void* any_t;

//#define asl_any_t         any_t
//#define asl_size_t        size_t

//#define asl_thread_t      thread_t
//#define asl_mutex_t       mutex_t
//#define asl_cond_t        cond_t

//typedef asl_any_t(*asl_thread_func_t)(asl_any_t);
//typedef void(*asl_thread_proc_t)(void);

// End of dependant code

// External variables
//extern int asl_debug;

class AslThreadManager;
class AslThread;
class AslMutex;

class AslMutex
{
	asl_mutex_t  m;
public:
	AslMutex(void);
	~AslMutex(void);

	int AslLock(void);
	int AslTryLock(void);
	int AslUnlock(void);

	asl_mutex_t* AslGetMutex(void);
};

class AslThreadManager
{
	int tidCount;			// Thread id counter

	asl_size_t stackSize;
	AslList<AslThread>  threadList;	// Thread list
	
	AslThread *AslGetThread(int);
	AslMutex thrmutex;
public:
	AslThreadManager(void);
	~AslThreadManager(void);

	void        AslSetStackSize(asl_size_t);
	asl_size_t  AslGetStackSize(void);

	void  AslSetConcurrency(int);
	int   AslGetConcurrency(void);
	
	int AslCreateThread(asl_thread_func_t, asl_any_t);
	int AslCreateThreadLWP(asl_thread_func_t, asl_any_t);
	int AslDeleteThread(int);
	
	int  AslGetThreadAnswer(int, asl_any_t*);
	void AslThreadExit(asl_any_t);
	
	void AslYieldProcessor(void);
	
	int AslKillThread(int);

	void AslStopThread(int);
	void AslContinueThread(int);
	int AslGetThreadID(void);
};

class AslThread
{
	int id;
	asl_thread_t thread;
	AslThreadManager *attrib;
	long int runMode;
public:
	AslThread(AslThreadManager*, int);
	~AslThread(void);


	int   AslGetThreadId(void);
	void  AslRunModeLWP(void);

	void  AslSetPriority(int);
	int   AslGetPriority(void);

	void   AslSuspendThread(void);
	void   AslContinueThread(void);	

	int   AslExecuteThread(asl_thread_func_t, asl_any_t);
	int   AslGetThreadAnswer(asl_any_t*);          
	
	asl_thread_t AslGetThread(void);
};

class AslCondition
{
	asl_cond_t c;
public:
	AslCondition(void);
	~AslCondition(void);

	int AslConditionWait(AslMutex*);
	int AslConditionBroadcast(void);
	int AslConditionSignal(void);
	asl_cond_t AslGetCondition(void); 
};

#endif
