/* SCCS  @(#)nsl_model.C	1.1---95/10/13--16:37:11 */
//
//	nsl_model.C
//

# include "nsl_all_kernel_include.h"

nsl_model::nsl_model() : nsl_public("nsl_model")
{
	init_model("");

}
nsl_model::nsl_model(const char* str,nsl_model* parent,int fg)
{
	init_model(str,parent,fg);
}
nsl_model::~nsl_model()
{
}
int nsl_model::init_model(const char* lname,nsl_model* parent,int fg)
{
	set_name(lname);
	set_type("model");
	set_base_type("model");

	DIFFM = new nsl_diff_manager; // initialize local diff manager
	DIFFM->set_m_parent(this);

	layer_list = new nsl_list("layer"); 
	module_list = new nsl_list("module"); 
	model_list = new nsl_list("model"); 

	layer_total = 0;
	module_total = 0;
	model_total = 0;

	m_children_list->append(layer_list);
	m_children_list->append(module_list);
	m_children_list->append(model_list);
   	m_children_list->append(DIFFM);

  	SYS_TIME = 0.0;	
	reset_time();
	DELTA = 0.1;
	set_end_time((float)1.0);

	if (SYSTEM == NULL)
	{
		SYSTEM = new nsl_system;
		NSL_SYSTEM = SYSTEM;
		cmd_out("nsl_model:Initializing SYSTEM...");
	}

	if (INTERPRETER == NULL)
	{
		INTERPRETER = new nsl_interpreter;
		NSL_INTERPRETER = INTERPRETER;
		cmd_out("nsl_model:Initializing NSL_INTERPRETER...");
	}

	if (NSL_DIFF == NULL)
	{
		NSL_DIFF = new nsl_diff;
		cmd_out("nsl_model:Initializing NSL_DIFF...");
 	}

	if (parent == 0) // fg == 1) 
	{
		SYSTEM->add_model(this);
//		INTERPRETER->add_object(this);
	}
	else
	{
		parent->add_model(this); // add this model to parent
	}

 	return 1;
}

// reset
void nsl_model::reset_time()
{
	CURRENT_TIME_STEP = 0;
	CURRENT_TIME = 0.0;
}
// delta
void nsl_model::set_delta(float i)
{
	DELTA = i;
}
// set
void nsl_model::set_time_step(int i)
{
	CURRENT_TIME_STEP = i;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}
void nsl_model::set_time(int i) // called from 'run'
{
	CURRENT_TIME_STEP = i;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}
void nsl_model::set_time(float i)
{
	CURRENT_TIME = i;
	CURRENT_TIME_STEP = (int) nsl_rint(CURRENT_TIME/DELTA);
}
// end
void nsl_model::set_end_time_step(int i)
{
	END_TIME_STEP = i;
	END_TIME = END_TIME_STEP*DELTA;
}
void nsl_model::set_end_time(float i)
{
	END_TIME = i;
	END_TIME_STEP = (int) nsl_rint(END_TIME/DELTA);
}
// increment
void nsl_model::incr_time()
{
	++CURRENT_TIME_STEP;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}

// add module
 
int nsl_model::add_module(nsl_module* module)
{
	module->set_index(++module_total);
	module_list->append(module);
	module->set_m_parent(this);

	return module_total;
}

// get module

nsl_module* nsl_model::get_module()
{
	if (module_list != NULL)
	{
	   module_list->wind_up(); 
	   nsl_module* module;
	      module = (nsl_module*)module_list->next(); // return first module
		return module;
	}
	else
	   cmd_out("NULL module_list");

	return (nsl_module*) 0;
}
nsl_module* nsl_model::get_module(const char* lname)
{
	if (module_list != NULL)
	{
	   module_list->wind_up();
	   const char* str;
	   nsl_module* module;
	   while (module = (nsl_module*)module_list->next())
	   {
		str = module->get_name();
		if (strcmp(lname,str) == 0)
			return (nsl_module*) module;
	   }
	}
	else
	   cmd_out("NULL 'nsl_module_list'");

	return (nsl_module*) 0;
}
nsl_module* nsl_model::get_module(const int lindex)
{
	if (module_list != NULL)
	{
	   module_list->wind_up();
	   nsl_module* module;
	   while (module = (nsl_module*)module_list->next())
	   {
		if (lindex == module->get_index())
			return (nsl_module*) module;
	   }
	}
	else
	   cmd_out("NULL 'nsl_module_list'");

	return (nsl_module*) 0;
}

// add model
 
int nsl_model::add_model(nsl_model* model)
{
	model->set_index(++model_total);
	model_list->append(model);
	model->set_m_parent(this);

	return model_total;
}

// get model

nsl_model* nsl_model::get_model()
{
	if (model_list != NULL)
	{
	   model_list->wind_up(); 
	   nsl_model* model;
	      model = (nsl_model*)model_list->next(); // return first model
		return model;
	}
	else
	   cmd_out("NULL model_list");

	return (nsl_model*) 0;
}

nsl_model* nsl_model::get_model(const char* fullstr)
{
	nsl_string mname;
	const char *str,*pname;

 	int mlen = strlen(fullstr); // full length

      	if ((str = (char *) strchr(fullstr,'.')) != NULL)
	{
		pname = str + 1; 	// path after first model name
		int plen = strlen(str); // extra path length
		strncpy(mname,fullstr,mlen-plen); // model name
		mname[mlen-plen]='\0';
	}
	else
	{
		strcpy(mname,fullstr);
		pname = 0;
	}

	nsl_model* model = (nsl_model*) model_list->get_by_name(mname);

	if (pname != NULL && model != NULL) 
		model = model->get_model(pname);

	return model;
}

nsl_model* nsl_model::get_model(const int lindex)
{
	if (model_list != NULL)
	{
	   model_list->wind_up();
	   nsl_model* model;
	   while (model = (nsl_model*)model_list->next())
	   {
		if (lindex == model->get_index())
			return (nsl_model*) model;
	   }
	}
	else
	   cmd_out("NULL 'nsl_model_list'");

	return (nsl_model*) 0;
}

// init module

int nsl_model::init()
{
	if (get_print_time_fg() == 1 || get_verbose_fg() == 1)
		cmd_out("// initiating model: ",get_name());

	if (module_list != NULL)
	{
	    module_list->wind_up();
	    nsl_module* m;
	    while (m = (nsl_module*) module_list->next()) 
		((nsl_module*) m)->proc(-1); // global
	}
//	else
//	   cmd_out("NULL 'nsl_module_list'");

	return 1;
}

// run module

// AW98 get rid of this and add global variable to ASL_SYS
extern void ASLrun(int);

int nsl_model::run()
{
//	if (get_print_time_fg() == 1 || get_verbose_fg() == 1)
//		cmd_out("// simulation time: ",get_time());

/* AW98	if (module_list != NULL)
	{
	    module_list->wind_up();
	    nsl_module* m;
	    while (m = (nsl_module*) module_list->next()) 
		((nsl_module*) m)->proc(CURRENT_TIME_STEP); // global
	}
	else
	   cmd_out("NULL 'nsl_module_list'");
*/
	ASLrun(CURRENT_TIME_STEP);

	return 1;
}
int nsl_model::run(const int i0,const int i1)
{
	int i;

 	if (i1 < i0)
	{
	    cerr << "BAD time interval: " << i0 << "," << i1 << "\n";
	    cerr.flush();
	    return 0;
	}

//	cmd_out("running from  i0 (time step): ",i0);
//	cmd_out("        until i1 (time step): ",i1);

	for (i = i0; i <= i1; i++)
	{
/*	    if (BREAKFG == 1)
	    {
		BREAKFG = 0;
		INTERRUPTFG = 1;
		cmd_error("Simulation Interrupt");
		return 1;
	    }
*/
	    set_time(i);
//	    cmd_out("// TimeStep: ",i);

 	    run();
	}
	return 1;
}
int nsl_model::run(const float t0,const float t1)
{
//	if (SIM_FG == 1)

	if (t1 < t0)
	{
	    cerr << "BAD time interval: " << t0 << "," << t1 << "\n";
	    cerr.flush();
	    return 0;
	}

//	cmd_out("running from  t0 (time): ",t0);
//	cmd_out("        until t1 (time): ",t1);

	float delta = get_delta();
	int i0 = (int) nsl_rint(t0/delta);
	int i1 = (int) nsl_rint(t1/delta);

	int mstatus = run(i0,i1);

	return mstatus;
}

// add layers

int nsl_model::add_layer(nsl_layer* layer)
{
	layer->set_index(++layer_total);
	layer_list->append(layer);
//	layer->set_m_parent(this);

	return layer_total;
}

// get layer

nsl_layer* nsl_model::get_layer()
{
	if (layer_list != NULL)
	{
	   layer_list->wind_up(); 
	   nsl_layer* layer;
	      layer = (nsl_layer*)layer_list->next(); // return first layer
		return layer;
	}
	else
	   cmd_out("NULL layer_list");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_layer(const char* lname)
{
	if (layer_list != NULL)
	{
	   layer_list->wind_up();
	   nsl_layer* layer;
	   const char* str;
	   while (layer = (nsl_layer*)layer_list->next())
	   {
		str = layer->get_name();
		if (strcmp(lname,str) == 0)
			return layer;
	   }
	}
	else
	   cmd_out("NULL 'layer_list'");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_layer(const int lindex)
{
	if (layer_list != NULL)
	{
	   layer_list->wind_up();
	   nsl_layer* layer;
	   while (layer = (nsl_layer*)layer_list->next())
	   {
		if (lindex == layer->get_index())
			return layer;
	   }
	}
	else
	   cmd_out("NULL 'layer_list'");

	return (nsl_layer*) 0;
}

void nsl_model::file_header(ostream& out)
{
	if (layer_list != NULL)
	{
	   out << "set delta " << DELTA << "\n";
	   out << "set end_time " << END_TIME << "\n";

	   out << "disable module ALL\n";
	   out << "disable layer ALL\n";

	   layer_list->wind_up();
	   nsl_layer* layer;
	   while (layer = (nsl_layer*)layer_list->next())
		if (layer->get_able_fg() == 1)
		    out << "enable layer " << layer->get_name() << "\n";
	}
	else
	   cmd_out("NULL 'layer_list'");
}

// print

int nsl_model::print_model_status(ostream& out) const
{
	print_base_status(out);
	out << "// layer_total:\t\t" << layer_total << "\n";
	out << "// module_total:\t" << module_total << "\n";
	out << "\n";
  	if (get_print_time_fg() == 1)
	   out << "set model print_time\tON\n";
	else // if (get_print_time_fg() == 0)
	   out << "set model print_time\t\tOFF\n";
	out << "set model delta\t\t" << DELTA << "\n";
	out << "set model current_time\t" << CURRENT_TIME << "\n";
	out << "set model end_time\t" << END_TIME << "\n";
	out.flush();
  	if (DIFFM != NULL)
		DIFFM->print_status(out);
	out.flush();

	return 1;
}
int nsl_model::print_status(ostream& out)  const // virtual
{
	print_model_status(out);
	return 1;
}
int nsl_model::print_model(ostream& out) const
{
//	print_base(out);
	print_model_status(out);
	return 1;
}
int nsl_model::print(ostream& out) const // virtual
{
	print_model(out);
	return 1;
}

// print all

int nsl_model::print_model_all_status(ostream& out) const
{
	print_model_status(out);
	print_base_all_status(out);
	return 1;
}
int nsl_model::print_all_status(ostream& out)  const // virtual
{
	print_model_all_status(out);
	return 1;
}
int nsl_model::print_model_all(ostream& out) const
{
	print_model(out);
	print_base_all(out);
	return 1;
}
int nsl_model::print_all(ostream& out) const // virtual
{
	print_model_all(out);
	return 1;
}

// read, write

int nsl_model::write_model(nsl_buffer& buf)
{
	nsl_list* list;

	if ((list = get_layer_list()) != NULL)
	{
           list->wind_up();
           nsl_layer* layer;
           while (layer = (nsl_layer*) list->next())
	   { 
		if (layer->get_file_able_fg() == 0)
			continue;

                buf << "// t=" << get_time() << "\n";
                buf << "assign layer " << layer->get_name() << "\n";
		layer->write(buf);
	    }
	    buf << "step\n";
	}
	else
	{
		cmd_out("Null model layer list");
	}
	return 1;
}
int nsl_model::write(nsl_buffer& buf)
{
	write_model(buf);
	return 1;
}
int nsl_model::read_model(nsl_buffer&)
{
	nsl_list* list;

	if ((list = get_layer_list()) != NULL)
	{
            list->wind_up();
            nsl_layer* layer;
            while (layer = (nsl_layer*) list->next())
	    { 
		if (layer->get_file_able_fg() == 0)
			continue;

//		layer->read(buf);
	    }
	}
	else
	{
		cmd_out("Null model layer list");
	}

	return 1;
}
int nsl_model::read(nsl_buffer& buf)
{
	read_model(buf);
	return 1;
}
