/* SCCS  @(#)nsl_matrix_ramp_library.C	1.1---95/08/13--20:48:17 */
//
//	nsl_matrix_ramp_library.C
//

# include "nsl_all_kernel_include.h"

// if a > 0 then a else 0

nsl_matrix NSLramp(const nsl_matrix& a) 
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > (num_type) 0)
			c[i][j] = a[i][j];
		else
			c[i][j] = 0;
	return c;
}

// if a > b then a - b else 0

nsl_matrix NSLramp(const nsl_matrix& a,const nsl_matrix& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b[i][j])
			c[i][j] = a[i][j] - b[i][j];
		else
			c[i][j] = 0;
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const nsl_data& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b)
			c[i][j] = a[i][j] - b.get_data();
		else
			c[i][j] = 0;
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const num_type b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b)
			c[i][j] = a[i][j] - b;
		else
			c[i][j] = 0;
	return c;
}

// if a > b then a - b + y2 else y1

nsl_matrix NSLramp(const nsl_matrix& a,const nsl_matrix& b,const nsl_matrix& y1,
	const nsl_matrix& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax() ||
	    a.get_imax() != y1.get_imax() || a.get_jmax() != y1.get_jmax() ||
	    a.get_imax() != y2.get_imax() || a.get_jmax() != y2.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("jmax3: ",y1.get_jmax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error("jmax4: ",y2.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b[i][j])
			c[i][j] = a[i][j] - b[i][j] + y2[i][j];
		else
			c[i][j] = y1[i][j];
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const nsl_matrix& b,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b[i][j])
			c[i][j] = a[i][j] - b[i][j] + y2;
		else
			c[i][j] = y1;
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const nsl_matrix& b,const nsl_data& y1,
	const nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b[i][j])
			c[i][j] = a[i][j] - b[i][j] + y2.get_data();
		else
			c[i][j] = y1.get_data();
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const num_type b,const num_type y1,
	const num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b)
			c[i][j] = a[i][j] - b + y2;
		else
			c[i][j] = y1;
	return c;
}
nsl_matrix NSLramp(const nsl_matrix& a,const nsl_data& b,const nsl_data& y1,
	const nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a[i][j] > b)
			c[i][j] = a[i][j] - b.get_data() + y2.get_data();
		else
			c[i][j] = y1.get_data();
	return c;
}
