/* SCCS  @(#)nsl_vector_step_library.C	1.1---95/08/13--20:48:24 */
//
//	nsl_vector_step_library.C
//

# include "nsl_all_kernel_include.h"

// if a > 0 then 1 else 0

nsl_vector NSLstep(const nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	for (int i = 0; i < s1; i++)
		if (a[i] > (num_type) 0)
			c[i] = (num_type) 1;
		else
			c[i] = 0;
	return c;
}

// if a > b then 1 else 0

nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
		if (a[i] > b[i])
			c[i] = (num_type) 1;
		else
			c[i] = 0;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	for (int i = 0; i < s1; i++)
		if (a[i] > b)
			c[i] = (num_type) 1;
		else
			c[i] = 0;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	for (int i = 0; i < s1; i++)
		if (a[i] > b)
			c[i] = (num_type) 1;
		else
			c[i] = 0;
	return c;
}

// if a > b then y2 else y1

nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const nsl_vector& y1,const nsl_vector& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
		if (a[i] > b[i])
			c[i] = y2[i];
		else
			c[i] = y1[i];
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
		if (a[i] > b[i])
			c[i] = y2;
		else
			c[i] = y1;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	for (int i = 0; i < s1; i++)
		if (a[i] > b[i])
			c[i] = y2.get_data();
		else
			c[i] = y1.get_data();
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const num_type b,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	for (int i = 0; i < s1; i++)
		if (a[i] > b)
			c[i] = y2;
		else
			c[i] = y1;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_data& b,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	for (int i = 0; i < s1; i++)
		if (a[i] > b)
			c[i] = y2.get_data();
		else
			c[i] = y1.get_data();
	return c;
}
